@@ -339,21 +339,25 @@ load_mapping(perl_yaml_loader_t *loader, char *tag)
     }
 
     /* Deal with possibly blessing the hash if the YAML tag has a class */
-    if (tag && strEQ(tag, TAG_PERL_PREFIX "hash"))
-        tag = NULL;
     if (tag) {
-        char *class;
-        char *prefix = TAG_PERL_PREFIX "hash:";
-        if (*tag == '!') {
-            prefix = "!";
+        if (strEQ(tag, TAG_PERL_PREFIX "hash")) {
+        }
+        else if (strEQ(tag, YAML_MAP_TAG)) {
+        }
+        else {
+            char *class;
+            char *prefix = TAG_PERL_PREFIX "hash:";
+            if (*tag == '!') {
+                prefix = "!";
+            }
+            else if (strlen(tag) <= strlen(prefix) ||
+                ! strnEQ(tag, prefix, strlen(prefix))
+            ) croak("%s",
+                loader_error_msg(loader, form("bad tag found for hash: '%s'", tag))
+            );
+            class = tag + strlen(prefix);
+            sv_bless(hash_ref, gv_stashpv(class, TRUE));
         }
-        else if (strlen(tag) <= strlen(prefix) ||
-            ! strnEQ(tag, prefix, strlen(prefix))
-        ) croak("%s",
-            loader_error_msg(loader, form("bad tag found for hash: '%s'", tag))
-        );
-        class = tag + strlen(prefix);
-        sv_bless(hash_ref, gv_stashpv(class, TRUE));
     }
 
     return hash_ref;
@@ -373,20 +377,24 @@ load_sequence(perl_yaml_loader_t *loader)
     while ((node = load_node(loader))) {
         av_push(array, node);
     }
-    if (tag && strEQ(tag, TAG_PERL_PREFIX "array"))
-        tag = NULL;
     if (tag) {
-        char *class;
-        char *prefix = TAG_PERL_PREFIX "array:";
-        if (*tag == '!')
-            prefix = "!";
-        else if (strlen(tag) <= strlen(prefix) ||
-            ! strnEQ(tag, prefix, strlen(prefix))
-        ) croak("%s",
-            loader_error_msg(loader, form("bad tag found for array: '%s'", tag))
-        );
-        class = tag + strlen(prefix);
-        sv_bless(array_ref, gv_stashpv(class, TRUE));
+        if (strEQ(tag, TAG_PERL_PREFIX "array")) {
+        }
+        else if (strEQ(tag, YAML_SEQ_TAG)) {
+        }
+        else {
+            char *class;
+            char *prefix = TAG_PERL_PREFIX "array:";
+            if (*tag == '!')
+                prefix = "!";
+            else if (strlen(tag) <= strlen(prefix) ||
+                ! strnEQ(tag, prefix, strlen(prefix))
+            ) croak("%s",
+                loader_error_msg(loader, form("bad tag found for array: '%s'", tag))
+            );
+            class = tag + strlen(prefix);
+            sv_bless(array_ref, gv_stashpv(class, TRUE));
+        }
     }
     return array_ref;
 }
@@ -400,24 +408,30 @@ load_scalar(perl_yaml_loader_t *loader)
     STRLEN length = (STRLEN)loader->event.data.scalar.length;
     char *anchor = (char *)loader->event.data.scalar.anchor;
     char *tag = (char *)loader->event.data.scalar.tag;
+    yaml_scalar_style_t style = loader->event.data.scalar.style;
     if (tag) {
-        char *class;
-        char *prefix = TAG_PERL_PREFIX "regexp";
-        if (strnEQ(tag, prefix, strlen(prefix)))
-            return load_regexp(loader);
-        prefix = TAG_PERL_PREFIX "scalar:";
-        if (*tag == '!')
-            prefix = "!";
-        else if (strlen(tag) <= strlen(prefix) ||
-            ! strnEQ(tag, prefix, strlen(prefix))
-        ) croak("%sbad tag found for scalar: '%s'", ERRMSG, tag);
-        class = tag + strlen(prefix);
-        scalar = sv_setref_pvn(newSV(0), class, string, strlen(string));
-        SvUTF8_on(scalar);
-    return scalar;
+        if (strEQ(tag, YAML_STR_TAG)) {
+            style = YAML_SINGLE_QUOTED_SCALAR_STYLE;
+        }
+        else {
+            char *class;
+            char *prefix = TAG_PERL_PREFIX "regexp";
+            if (strnEQ(tag, prefix, strlen(prefix)))
+                return load_regexp(loader);
+            prefix = TAG_PERL_PREFIX "scalar:";
+            if (*tag == '!')
+                prefix = "!";
+            else if (strlen(tag) <= strlen(prefix) ||
+                ! strnEQ(tag, prefix, strlen(prefix))
+            ) croak("%sbad tag found for scalar: '%s'", ERRMSG, tag);
+            class = tag + strlen(prefix);
+            scalar = sv_setref_pvn(newSV(0), class, string, strlen(string));
+            SvUTF8_on(scalar);
+            return scalar;
+        }
     }
 
-    if (loader->event.data.scalar.style == YAML_PLAIN_SCALAR_STYLE) {
+    else if (style == YAML_PLAIN_SCALAR_STYLE) {
         if (strEQ(string, "~"))
             return newSV(0);
         else if (strEQ(string, ""))
@@ -462,7 +476,7 @@ load_scalar(perl_yaml_loader_t *loader)
 
     scalar = newSVpvn(string, length);
 
-    if (loader->event.data.scalar.style == YAML_PLAIN_SCALAR_STYLE && looks_like_number(scalar) ) {
+    if (style == YAML_PLAIN_SCALAR_STYLE && looks_like_number(scalar) ) {
         /* numify */
         SvIV_please(scalar);
     }
